@@ -13,7 +13,6 @@ void		display_editor_help();
 void		*abook_malloc(size_t size);
 void		*abook_realloc(void *ptr, size_t size);
 FILE		*abook_fopen (const char *path, const char *mode);
-void		trim_line(char *str); /* from conff.c */
 void		quit_abook();
 #ifdef _AIX
 int		strcasecmp (const char *, const char *);
@@ -50,5 +49,12 @@ int		strncasecmp (const char *, const char *, size_t);
 #       define max(x,y) (((x)>(y)) ? (x):(y))
 #endif
 
+#define ISSPACE(c)	isspace((unsigned char)c)
+
+#ifndef DEBUG
+#	define NDEBUG	1
+#else
+#	undef NDEBUG
+#endif
 
 #endif
