@@ -29,6 +29,10 @@ list_item *database = NULL;
 
 int items = 0;
 
+#define INITIAL_LIST_CAPACITY	30
+
+int list_capacity = 0;
+
 extern int first_list_item;
 extern int curitem;
 extern char *selected;
@@ -68,13 +72,10 @@ struct abook_field abook_fields[ITEM_FIELDS] = {
 };
 
 
-
-#define MAX_LINESIZE	400
-
 int
 parse_database(FILE *in)
 {
-        char line[MAX_LINESIZE];
+        char *line = NULL;
 	char *tmp;
 	int sec=0, i;
  	list_item item;
@@ -82,7 +83,7 @@ parse_database(FILE *in)
 	memset(&item, 0, sizeof(item));
 	
 	for(;;) {
-		fgets(line, MAX_LINESIZE, in);
+		line = getaline(in);
 		if( feof(in) ) {
 			if( item[NAME] && sec )
 				add_item2database(item);
@@ -90,11 +91,11 @@ parse_database(FILE *in)
 				free_list_item(item);
 			break;
 		}
-		trim_line(line);
 
-		if( !*line || *line == '\n' || *line == '#' )
+		if( !*line || *line == '\n' || *line == '#' ) {
+			free(line);
 			continue;
-		else if( *line == '[' ) {
+		} else if( *line == '[' ) {
 			if( item[NAME] && sec )
 				add_item2database(item);
 			else
@@ -104,13 +105,15 @@ parse_database(FILE *in)
 			if ( !(tmp = strchr(line, ']')))
 				sec = 0; /*incorrect section lines are skipped*/
 		} else if((tmp = strchr(line, '=') ) && sec ) {
-			*tmp = '\0'; tmp++;
+			*tmp++ = '\0';
 			for(i=0; i<ITEM_FIELDS; i++)
 				if( !strcmp(abook_fields[i].key, line) )
 					item[i] = strdup(tmp);
 		}
+		free(line);
 	}
 
+	free(line);
 	return 0;
 }
 
@@ -213,6 +216,7 @@ close_database()
 
 	items = 0;
 	first_list_item = curitem = -1;
+	list_capacity = 0;
 }
 
 #define _MAX_FIELD_LEN(X)	(X == EMAIL ? MAX_EMAILSTR_LEN:MAX_FIELD_LEN)
@@ -235,6 +239,24 @@ validate_item(list_item item)
 		}
 }
 
+
+static void
+adjust_list_capacity()
+{
+	if(list_capacity < 1)
+		list_capacity = INITIAL_LIST_CAPACITY;
+	else if(items >= list_capacity)
+		list_capacity *= 2;
+	else if(list_capacity / 2 > items)
+		list_capacity /= 2;
+	else
+		return;
+
+	database = abook_realloc(database,
+			sizeof(list_item) * list_capacity);
+	selected = abook_realloc(selected, list_capacity);
+}
+
 int
 add_item2database(list_item item)
 {
@@ -243,9 +265,8 @@ add_item2database(list_item item)
 		return 1;
 	}
 
-	database = abook_realloc(database,
-			sizeof(list_item) * ++items);
-	selected = abook_realloc(selected, items);
+	if( ++items > list_capacity)
+		adjust_list_capacity();
 
 	validate_item(item);
 
@@ -290,53 +311,22 @@ remove_items()
 		curitem = LAST_ITEM;
 
 
-	if(items > 0) {
-		database = abook_realloc(database,
-				sizeof(list_item) * items);
-		selected = abook_realloc(selected, items);
-	} else
-		close_database();
+	adjust_list_capacity();
 
 	select_none();
 	clear_statusline();	
 	refresh_list();
 }
 
-void
-sort_surname()
-{
-	char s1[80], s2[80];
-	int i, j, min;
-        list_item t;
-
-	select_none();
-
-	for( i = 0; i < LAST_ITEM; i++) {
-		min = i;
-		for( j = i + 1; j < items; j++ ) {
-			get_surname(j, s1);
-			get_surname(min, s2);
-#ifdef DEBUG
-			fprintf(stderr, 
-				"sort_surname(): 1 = %s 2 = %s\n", s1, s2);
-#endif
-			if( strcmp( s1, s2 ) < 0 )
-				min = j;
-		}
-                itemcpy(t, database[min]);
-		itemcpy(database[min], database[i]);
-		itemcpy(database[i], t);
-	}
-	refresh_screen();
-}
-
-void
-get_surname(int item, char *name)
+char *
+get_surname(char *s)
 {
 	int i, a;
+	int len = strlen(s);
+	char *name = strdup(s);
 
-	for( a = 0, i = strlen(database[item][NAME]) - 1; i >= 0; i--, a++ ) {
-		name[a] = database[item][NAME][i];
+	for( a = 0, i = len - 1; i >= 0; i--, a++ ) {
+		name[a] = s[i];
 		if(name[a] == ' ')
 			break;
 	}
@@ -344,27 +334,60 @@ get_surname(int item, char *name)
 	name[ a ] = 0;
 
 	revstr(name);
+
+	return name;
 }
 
+static int
+surnamecmp(const void *i1, const void *i2)
+{
+	int ret;
+	list_item a,b;
+	char *s1, *s2;
+
+	itemcpy(a, i1);
+	itemcpy(b, i2);
+
+	s1 = get_surname(a[NAME]);
+	s2 = get_surname(b[NAME]);
+
+	if( !(ret = safe_strcmp(s1, s2)) )
+		ret = safe_strcmp(a[NAME], b[NAME]);
+
+	free(s1);
+	free(s2);
+
+	return ret;
+}
+
+static int
+namecmp(const void *i1, const void *i2)
+{
+	list_item a, b;
+
+	itemcpy(a, i1);
+	itemcpy(b, i2);
+	
+	return safe_strcmp( a[NAME], b[NAME] );
+}
 
 void
 sort_database()
 {
-	int i, j, min;
-	list_item t;
-
 	select_none();
 	
-        for( i = 0; i < LAST_ITEM; i++) {
-		min = i;
-		for( j = i + 1; j < items; j++ )
-			if( strcmp( database[j][NAME], database[min][NAME] ) < 0 )
-				min = j;
+	qsort((void *)database, items, sizeof(list_item), namecmp);
+
+	refresh_screen();
+}
+
+void
+sort_surname()
+{
+	select_none();
+
+	qsort((void *)database, items, sizeof(list_item), surnamecmp);
 
-		itemcpy(t, database[min]);
-		itemcpy(database[min], database[i]);
-		itemcpy( database[i], t);
-	}
 	refresh_screen();
 }
 
@@ -427,7 +450,7 @@ void
 print_number_of_items()
 {
 	char *str =
-		mkstr("|%3d/%3d", selected_items(), items);
+		mkstr("     |%3d/%3d", selected_items(), items);
 
 	mvaddstr(0, COLS-strlen(str), str);
 
@@ -457,7 +480,7 @@ void
 print_database()
 {
 	FILE *handle;
-	char *command = options_get_str("print", "command");
+	char *command = options_get_str("print_command");
 
 	statusline_addstr("Print addressbook? (y/N)");
 	switch( getch() ) {
@@ -468,15 +491,11 @@ print_database()
 	}
 	clear_statusline();
 
-	if( ! *command || (handle = popen(command, "w")) == NULL) {
-		free(command);
+	if( ! *command || (handle = popen(command, "w")) == NULL)
 		return;
-	}
 
 	fexport("text", handle);
 	
 	pclose(handle);
-
-	free(command);
 }
 
