@@ -7,20 +7,22 @@
 #define MAX_EMAILSTR_LEN	(MAX_EMAILS*MAX_EMAIL_LEN + MAX_EMAILS + 1)
 #define MAX_FIELD_LEN		81
 
-#define NAME			0
-#define EMAIL			1
-#define ADDRESS			2
-#define CITY			3
-#define STATE			4
-#define ZIP			5
-#define COUNTRY			6
-#define PHONE			7
-#define WORKPHONE		8
-#define FAX			9
-#define MOBILEPHONE		10
-#define NICK			11
-#define URL			12
-#define NOTES			13
+enum {
+	NAME,
+	EMAIL,
+	ADDRESS,
+	CITY,
+	STATE,
+	ZIP,
+	COUNTRY,
+	PHONE,
+	WORKPHONE,
+	FAX,
+	MOBILEPHONE,
+	NICK,
+	URL,
+	NOTES
+};
 
 #define LAST_FIELD		NOTES
 
@@ -48,7 +50,7 @@ void		clear_database();
 void		find(int next);
 void		print_number_of_items();
 void		read_database();
-void		get_surname(int item, char *name);
+char		*get_surname(char *s);
 void		print_database();
 
 #define LAST_ITEM	(items - 1)
