@@ -41,8 +41,7 @@ editor_tab(int tab)
 		"/  OTHER  \\"
 	};
 
-	for(i=0; i < EDITW_COLS; i++)
-		mvwaddch(editw, TABLINE +1, i, '-');
+	mvwhline(editw, TABLINE+1, 0, ACS_HLINE, EDITW_COLS);
 
 	for(i=0; i < TABS; i++)
 		mvwaddstr(editw, TABLINE, 16 * i + 3, tab_names[i]);
@@ -104,22 +103,26 @@ close_editor()
 static void
 print_editor_header(int item)
 {
-	char header[EDITW_COLS];
+	char *header;
 	char email[MAX_EMAIL_LEN];
 	int i, x, len;
+	header = (char *)malloc(EDITW_COLS);
 
 	get_first_email(email, item);
 	
 	if( snprintf(header, EDITW_COLS, "%s <%s>", database[item][NAME],
 				email ) ==  -1 || !*database[item][EMAIL] )
-	if( snprintf(header, EDITW_COLS, "%s", database[item][NAME]) == -1)
+	if( snprintf(header, EDITW_COLS, "%s", database[item][NAME]) == -1) {
+		free(header);
 		return;
+	}
 
 	len = strlen(header);
 	x = (EDITW_COLS - len) / 2;
 	mvwaddstr(editw, 0, x, header);
 	for(i = x; i < x + len; i++)
 		mvwaddch(editw,1, i, '^');
+	free(header);
 }
 
 static void
