@@ -13,10 +13,14 @@ void		add_item();
 #define EDITOR_HELPLINE	"?:help c:contact a:address p:phone o:other"
 
 #define TABLINE		2
-#define TAB_CONTACT	0
-#define TAB_ADDRESS	1
-#define TAB_PHONE	2
-#define TAB_OTHER	3
+
+enum {
+	TAB_CONTACT,
+	TAB_ADDRESS,
+	TAB_PHONE,
+	TAB_OTHER
+};
+
 #define MAX_TAB		TAB_OTHER
 	
 #define TABS		(MAX_TAB+1)
