@@ -120,7 +120,7 @@ wenter_string(WINDOW *win, const int maxlen, const int flags)
 			continue;
 		str[i++] = ch;
 		waddch(win,ch);
-		if( i+1 > size )
+		if( i + 1 > size )
 			str = REALLOC( str, size *= 2 );
 		if( maxlen > 0 && i > maxlen)
 			break;
@@ -129,7 +129,7 @@ out:
 	if( i > 0 && str != NULL )
 		str[i] = 0;
 	
-	if(i==0)
+	if(i == 0)
 		*str=0;
 
 	return str;
@@ -181,10 +181,10 @@ my_getcwd()
 	char *dir = NULL;
 	int size = 100;
 
-	dir = malloc(size);
+	dir = MALLOC(size);
 
 	while( getcwd(dir, size) == NULL && errno == ERANGE )
-		dir = realloc(dir, size *= 2);
+		dir = REALLOC(dir, size *= 2);
 
 	return dir;
 }
@@ -202,6 +202,8 @@ int filesel_first_list_item = -1;
 int filesel_items=0;
 struct filesel_list_item *lst = NULL ;
 
+int filesel_list_capacity = 0;
+
 WINDOW *filesel_list;
 
 static void
@@ -211,6 +213,7 @@ filesel_close_list()
 	lst = NULL;
 	filesel_items = 0;
 	filesel_first_list_item = filesel_curitem = -1;
+	filesel_list_capacity = 0;
 }
 
 static void
@@ -256,7 +259,13 @@ filesel_check_list()
 static int
 filesel_add_filesel_list_item(struct filesel_list_item item)
 {
-	lst = realloc(lst, ++filesel_items * sizeof(struct filesel_list_item) );
+	if( ++filesel_items > filesel_list_capacity) {
+		filesel_list_capacity =
+		filesel_list_capacity < 1 ? 30:filesel_list_capacity << 1;
+		lst = REALLOC(lst, filesel_list_capacity *
+			sizeof(struct filesel_list_item) );
+	}
+
 	lst[FILESEL_LAST_ITEM] = item;
 
 	filesel_check_list();
@@ -274,12 +283,18 @@ filesel_add_filesel_list_item(struct filesel_list_item item)
 #	endif
 #endif
 
+#ifdef FILESEL_ALPHABETIC_ORDER
+static int
+filenamecmp(const void *a, const void *b)
+{
+	return strcmp( ((struct filesel_list_item *)a) -> filename,
+			((struct filesel_list_item *)b) -> filename );
+}
+#endif
+
 static void
 filesel_sort_list()
 {
-#ifdef FILESEL_ALPHABETIC_ORDER
-	int min;
-#endif
 #ifdef FILESEL_DIRS_FIRST
         int i, j, fdp=0;
         struct filesel_list_item tmp;
@@ -288,43 +303,29 @@ filesel_sort_list()
         	if( ++fdp >= FILESEL_LAST_ITEM )
                 	return;
 
-	for(i = fdp +1; i < filesel_items; i++) {
+	for(i = fdp + 1; i < filesel_items; i++) {
 		if(lst[i].type == FLSL_TYPE_DIR ) {
-			tmp=lst[fdp];
-			lst[fdp++]=lst[i];
+			tmp = lst[fdp];
+			lst[fdp++] = lst[i];
 			for( j = i; j > fdp ; j--)
-				lst[j] = lst[j-1];
-			lst[fdp]=tmp;
+				lst[j] = lst[j - 1];
+			lst[fdp] = tmp;
 		}
 	}
 #endif
 #ifdef FILESEL_ALPHABETIC_ORDER
 #ifdef ABOOK_SRC
-	if( options_get_int("filesel", "sort") ) {
+	if( options_get_int("filesel_sort") ) {
 #endif
         if(fdp > 1)
-        for( i = 0; i < fdp - 2; i++) {
+		qsort((void *)lst, fdp, sizeof(struct filesel_list_item),
+				filenamecmp );
 
-                min = i;
-                for( j = i + 1; j < fdp - 1; j++ )
-                        if( strcmp( lst[j].filename, lst[min].filename ) < 0 )
-                                min = j;
-
-                tmp = lst[min];
-                lst[min] = lst[i];
-                lst[i] = tmp;
-        }
-
-        for( i = fdp; i < FILESEL_LAST_ITEM; i++) {
-                min = i;
-                for( j = i + 1; j < filesel_items; j++ )
-                        if( strcmp( lst[j].filename, lst[min].filename ) < 0 )
-                                min = j;
-
-                tmp = lst[min];
-                lst[min] = lst[i];
-                lst[i] = tmp;
-        }
+	qsort((void *)lst + fdp * sizeof(struct filesel_list_item),
+			FILESEL_LAST_ITEM - fdp + 1,
+			sizeof(struct filesel_list_item),
+			filenamecmp );
+ 
 #ifdef ABOOK_SRC
 	}
 #endif
@@ -363,7 +364,7 @@ filesel_print_list_line(int i, int line)
 	if( lst[i].type == FLSL_TYPE_DIR )
 		wattron(filesel_list, A_BOLD);
 	
-	mvwaddstr(filesel_list, line, 5, lst[i].filename);
+	mvwaddnstr(filesel_list, line, 5, lst[i].filename, COLS - 5);
 }
 
 static void
@@ -518,7 +519,7 @@ filesel_enter()
 
 	if(lst[filesel_curitem].type == FLSL_TYPE_DIR) {
 		dir = my_getcwd();
-		newdir = malloc(strlen(dir)+strlen(lst[filesel_curitem].filename) +2 );
+		newdir = MALLOC(strlen(dir)+strlen(lst[filesel_curitem].filename) +2 );
 		strcpy(newdir, dir);
 		strcat(newdir, "/");
 		strcat(newdir,lst[filesel_curitem].filename);
@@ -594,7 +595,7 @@ filesel()
 
 	if( !filesel_loop() ) {
 		dir = my_getcwd();
-		tmp = malloc(strlen(dir) + strlen(lst[filesel_curitem].filename) + 2);
+		tmp = MALLOC(strlen(dir) + strlen(lst[filesel_curitem].filename) + 2);
 		strcpy(tmp,dir);
 		strcat(tmp, "/");
 		strcat(tmp, lst[filesel_curitem].filename);
