@@ -582,7 +582,7 @@ mutt_read_line(FILE *in, char **alias, char **rest)
 	fgets(line, LINESIZE, in);
 	remove_newlines(line);
 
-	while( isspace(*ptr) )
+	while( ISSPACE(*ptr) )
 		ptr++;
 
 	if( strncmp("alias", ptr, 5) ) {
@@ -591,19 +591,19 @@ mutt_read_line(FILE *in, char **alias, char **rest)
 		
 	ptr += 5;
 
-	while( isspace(*ptr) )
+	while( ISSPACE(*ptr) )
 		ptr++;
 
 	tmp = ptr;
 
-	while( ! isspace(*ptr) )
+	while( ! ISSPACE(*ptr) )
 		ptr++;
 
 	*alias = malloc(ptr-tmp+1);
 	strncpy(*alias, tmp, ptr-tmp);
 	*(*alias+(ptr-tmp)) = 0;
 
-	while( isspace(*ptr) )
+	while( ISSPACE(*ptr) )
 		ptr++;
 
 	*rest = strdup(ptr);	
@@ -630,7 +630,7 @@ mutt_parse_email(char *mutt_item[3])
 	tmp = mutt_item[MUTT_NAME];
 
 	for(i=strlen(tmp)-1; i>0; i--)
-		if(isspace(tmp[i]))
+		if(ISSPACE(tmp[i]))
 			tmp[i] = 0;
 		else
 			break;
@@ -750,7 +750,7 @@ ldif_export_database(FILE *out)
  * html export filter
  */
 
-static void            html_export_write_head(FILE *out);
+static void            html_export_write_head(FILE *out, int extra_column);
 static void            html_export_write_tail(FILE *out);
 
 static int
@@ -758,11 +758,15 @@ html_export_database(FILE *out)
 {
 	char tmp[MAX_EMAILSTR_LEN];
 	int i;
+	int extra_column = options_get_int("extra_column");
 
 	if( items < 1 )
 		return 2;
-	
-	html_export_write_head(out);
+
+	extra_column = (extra_column > 2 && extra_column < ITEM_FIELDS) ?
+		extra_column : PHONE;
+
+	html_export_write_head(out, extra_column);
 	for( i = 0; i < items; i++ ) {
 		get_first_email(tmp, i);
 		fprintf(out, "<tr>\n<td><a href=\"mailto:%s\">%s</a>\n",
@@ -771,7 +775,7 @@ html_export_database(FILE *out)
 				
 		fprintf(out, "<td>%s\n<td>%s\n",
 				database[i][EMAIL],
-				safe_str(database[i][PHONE]) );
+				safe_str(database[i][extra_column]) );
 		fprintf(out, "</tr>\n\n");
 	}
 	html_export_write_tail(out);
@@ -781,11 +785,9 @@ html_export_database(FILE *out)
 
 
 static void
-html_export_write_head(FILE *out)
+html_export_write_head(FILE *out, int extra_column)
 {
-	char *realname;
-
-	realname = get_real_name();
+	char *realname = get_real_name();
 
 	fprintf(out, "<html>\n<head>\n	<title>%s's addressbook</title>",
 			realname );
@@ -794,7 +796,8 @@ html_export_write_head(FILE *out)
 	fprintf(out, "<br><br>\n\n");
 
 	fprintf(out, "<center><table border>\n");
-	fprintf(out, "\n<tr><th>Name<th>E-mail address(es)<th>Phone</tr>\n\n");
+	fprintf(out, "\n<tr><th>Name<th>E-mail address(es)<th>%s</tr>\n\n",
+			abook_fields[extra_column].name);
 
 	free(realname);
 }
@@ -998,13 +1001,13 @@ gcrd_export_database(FILE *out)
 {
 	char emails[MAX_EMAILS][MAX_EMAIL_LEN];
 	int i, j;
-	char name[MAX_FIELD_LEN];
+	char *name;
 
 	for(i=0; i < items; i++ ) {
 		fprintf(out, "BEGIN:VCARD\nFN:%s\n",
 				safe_str(database[i][NAME]));
 
-		get_surname(i, name);
+		name = get_surname(database[i][NAME]);
 	        for( j = strlen(database[i][NAME]) - 1; j >= 0; j-- ) {
 	                if(database[i][NAME][j] == ' ')
 	                        break;
@@ -1015,6 +1018,8 @@ gcrd_export_database(FILE *out)
 			safe_str(database[i][NAME])
 			); 
 
+		free(name);
+
 		if ( database[i][ADDRESS] )
 			fprintf(out, "ADR:;;%s;%s;%s;%s;%s\n",
 				safe_str(database[i][ADDRESS]),
@@ -1117,7 +1122,7 @@ text_export_database(FILE * out)
 	char emails[MAX_EMAILS][MAX_EMAIL_LEN];
 	int i, j;
 	char *realname = get_real_name();
-	char *style = options_get_str("address", "style");
+	char *style = options_get_str("address_style");
 
 	fprintf(out,
 		"-----------------------------------------\n%s's address book\n"
@@ -1201,8 +1206,6 @@ text_export_database(FILE * out)
 
 	fprintf(out, "-----------------------------------------\n");
 
-	free(style);
-	
 	return 0;
 }
 
