@@ -84,7 +84,7 @@ void
 print_list_line(int i, int line)
 {
 	char tmp[MAX_EMAILSTR_LEN];
-	int extra_column = options_get_int("list", "extra_column");
+	int extra_column = options_get_int("extra_column");
 	int real_emaillen = (extra_column > 2 && extra_column < ITEM_FIELDS) ?
 		EMAILLEN : EMAILPOS - COLS;
 
@@ -94,7 +94,7 @@ print_list_line(int i, int line)
 		mvwaddch(list, line, 0, '*' );
 	
 	mvwaddnstr(list, line, NAMEPOS, database[i][NAME], NAMELEN);
-	if( options_get_int( "list", "show_all_emails"  ) )
+	if( options_get_int( "show_all_emails"  ) )
 		mvwaddnstr(list, line, EMAILPOS, database[i][EMAIL],
 				real_emaillen);
 	else {
@@ -114,7 +114,7 @@ print_list_line(int i, int line)
 void
 list_headerline()
 {
-	int extra_column = options_get_int("list", "extra_column");
+	int extra_column = options_get_int("extra_column");
 
 	attrset(A_BOLD);
 	mvaddstr(2, NAMEPOS, abook_fields[NAME].name);
