@@ -20,6 +20,14 @@
 #	include "abook.h"
 #endif
 
+#ifndef DEBUG
+#	define NDEBUG	1
+#else
+#	undef NDEBUG
+#endif
+
+#include <assert.h>
+
 char *
 revstr(char *str)
 {
@@ -59,6 +67,26 @@ strlower(char *str)
 	return tmp;
 }
 
+char *
+strtrim(char *s)
+{
+	char *t, *tt;
+
+	assert(s != NULL);
+
+	for(t = s; ISSPACE(*t); t++);
+
+	memmove(s, t, strlen(t)+1);
+
+	for (tt = t = s; *t != '\0'; t++)
+		if(!ISSPACE(*t))
+			tt = t+1;
+
+	*tt = '\0';
+
+	return s;
+}
+
 
 #ifdef HAVE_CONFIG_H
 #	include "config.h"
@@ -167,11 +195,72 @@ my_getcwd()
 	dir = malloc(size);
 	
 	while( getcwd(dir, size) == NULL && errno == ERANGE )
-		dir = realloc(dir, size *= 2);
+		dir = realloc(dir, size *=2);
 
 	return dir;
 }
 
+#define INITIAL_SIZE	128
+#ifndef ABOOK_SRC
+#	define abook_malloc(X) malloc(X)
+#	define abook_realloc(X, XX) realloc(X, XX)
+#endif
+
+char *
+getaline(FILE *f)
+{
+	char *buf;		/* buffer for line */
+	size_t size;		/* size of buffer */
+	size_t inc;		/* how much to enlarge buffer */
+	size_t len;		/* # of chars stored into buf before '\0' */
+	char *p;
+	const size_t thres = 128; /* initial buffer size (most lines should
+				     fit into this size, so think of this as
+				     the "long line threshold").  */
+	const size_t mucho = 128; /* if there is at least this much wasted
+				     space when the whole buffer has been
+				     read, try to reclaim it.  Don't make
+				     this too small, else there is too much
+				     time wasted trying to reclaim a couple
+				     of bytes.  */
+	const size_t mininc = 64; /* minimum number of bytes by which
+				     to increase the allocated memory */
+
+	len = 0;
+	size = thres;
+	buf = abook_malloc(size);
+
+	while (fgets(buf+len, size-len, f) != NULL) {
+		len += strlen(buf+len);
+		if (len > 0 && buf[len-1] == '\n')
+			break;		/* the whole line has been read */
+
+		for (inc = size, p = NULL; inc > mininc; inc /= 2)
+			if ((p = abook_realloc(buf, size + inc)) != NULL)
+				break;
+
+		size += inc;
+		buf = p;
+	}
+
+	if (len == 0) {
+		free(buf);
+		return NULL;	/* nothing read (eof or error) */
+	}
+
+	if (buf[len-1] == '\n')	/* remove newline, if there */
+		buf[--len] = '\0';
+
+	if (size - len > mucho) { /* a plenitude of unused memory? */
+		p = abook_realloc(buf, len+1);
+		if (p != NULL) {
+			buf = p;
+			size = len+1;
+		}
+	}
+
+	return buf;
+}
 
 /**************************************************************
  * Original:
